package org.apidesign.apifest08.currency;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Currency;

/**
 * Default implementation of {@link Money} interface. This class is immutable.
 * @author lukas
 *
 */
public final class MoneyImpl implements Serializable, Money{
	private static final long serialVersionUID = -6091808475616516136L;

	private final BigDecimal amount;
	
	private final Currency currency;

	public MoneyImpl(BigDecimal amount, Currency currency) {
		if (amount==null) throw new NullPointerException("Amount is null");
		if (currency==null) throw new NullPointerException("Currency is null"+currency);
		this.amount = amount;
		this.currency = currency;
	}
	
	public MoneyImpl(long amount, Currency currency) {
		this(BigDecimal.valueOf(amount), currency);
	}
	
	public MoneyImpl(double amount, Currency currency) {
		this(BigDecimal.valueOf(amount), currency);
	}

	/**
	 * Returns amount.
	 * @return
	 */
	public BigDecimal getAmount() {
		return amount;
	}
	
	/**
	 * Returns currency.
	 */
	public Currency getCurrency() {
		return currency;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((amount == null) ? 0 : amount.hashCode());
		result = prime * result
				+ ((currency == null) ? 0 : currency.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (!(obj instanceof MoneyImpl))
			return false;
		MoneyImpl other = (MoneyImpl) obj;
		if (amount == null) {
			if (other.amount != null)
				return false;
		} else if (amount.compareTo(other.amount)!=0)
			return false;
		if (currency == null) {
			if (other.currency != null)
				return false;
		} else if (!currency.equals(other.currency))
			return false;
		return true;
	}
	
	@Override
	public String toString() {
		return getClass().getName()+"["+currency+amount+"]";
	}

	
	
}
